<?php

/**
 * InterWorx Hosting Control Panel
 *
 * <pre>
 * +----------------------------------------------------------------------+
 * | Copyright (c) 2000-2010 InterWorx L.L.C., All Rights Reserved.       |
 * +----------------------------------------------------------------------+
 * | Redistribution and use in source form, with or without modification  |
 * | is NOT permitted without consent from the copyright holder.          |
 * |                                                                      |
 * | THIS SOFTWARE IS PROVIDED BY THE AUTHOR AND CONTRIBUTORS "AS IS" AND |
 * | ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO,    |
 * | THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A          |
 * | PARTICULAR PURPOSE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL,    |
 * | EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO,  |
 * | PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR   |
 * | PROFITS; OF BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY  |
 * | OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT         |
 * | (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE    |
 * | USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH     |
 * | DAMAGE.                                                              |
 * +----------------------------------------------------------------------+
 * </pre>
 *
 * @package InterWorx
 * @author  Malin Lorch <mlorch@liquidweb.com>
 * @date    Apr 20, 2020
 */

/**
 * Backup Plan class.
 *
 * @package Interworx
 * @subpackage Plugin_Acronis
 */
class Model_Backups_BackupPlan extends Model_Acronis
{
    const NEXT_BACKUP = 'nextBackup';
    const LAST_BACKUP = 'lastBackup';
    const LAST_STATUS = 'lastStatus';
    const ENABLED = 'enabled';
    const POLICY_NAME = 'name';
    const RUNNING = 'running';
    const STATUS = 'status';
    const DEFAULT_PLAN_NAME = 'InterWorx Default';

    private $_Application;

    /**
     * Check if any plan is assigned to the resource.
     *
     * @return boolean
     */
    public function isProtected(): bool
    {
        // checking if policy we are using (could be cached) is protecting resource
        // rather than if ANY plan is protecting the resource.
        try {
            $Policy = $this->_requestRootPolicy();
            return $Policy->isProtectingResource($this->getInstanceId());
        } catch (Exception_Acronis_BackupPlan_NoApplicablePolicies $E) {
            return false;
        }
    }


    /**
     * Get the policy in use by the plugin.
     *
     * @param string $policy_name
     * @return Model_Backups_Plan_Policy_Total
     */
    public function getPolicyByName($policy_name): Model_Backups_Plan_Policy_Total
    {
        $policies = $this->getPoliciesApi()->getApplicableRootPolicies($this->getInstanceId());

        foreach ($policies as $Policy) {
            if ($Policy->getName() === $policy_name) {
                return $Policy;
            }
        }

        throw new Exception_Acronis_BackupPlan_Missing('Policy with name does not exist: ' . $policy_name);
    }

    /**
     * Send api request to retrieve policy (plan template)
     *
     * Will first check if cached policy exists, then applied policies, and lastly applicable policy.
     *
     * @return Model_Backups_Plan_Policy_Total
     */
    private function _requestRootPolicy(): Model_Backups_Plan_Policy_Total
    {
        $Api =  $this->getPoliciesApi();
        $Cache = $this->_getCache();

        // first check if cached plan exists - want to use regardless if applied or not
        // e.g. in case protection is turned off, when they turn it on we want to make sure it's
        // the same plan.
        if ($Cache->isCached($Cache::CACHE_PLAN_ID)) {
            $Policy = $this->_getCachedPolicy();
        }

        if (!isset($Policy)) {
            $policies = $Api->getAppliedRootPolicies();

            if (empty($policies)) {
                IW::getLog()->debug("Acronis: resource is not protected by any plan. Attempting to find applicable policy");
                $policies = $Api->getApplicableRootPolicies();
            }

            $Policy = $this->_getMostApplicableRootPolicy($policies);
        }

        IW::getLog()->debug("Acronis: " . __FUNCTION__ . ": returning plan with name: " . $Policy->getName());
        return $Policy;
    }

    /**
     * Check id in cache - get policy if exists.
     *
     * Will sort out id mismatch, ie if the stored plan id is of the wrong type.
     */
    private function _getCachedPolicy()
    {
        $Api =  $this->getPoliciesApi();
        $Cache = $this->_getCache();
        $cache_id = $Cache->getPlanId();

        try {
            $Policy = $Api->getTotalPolicy($cache_id);
            return $Policy;
        } catch (Exception_Acronis_BackupPlan_WrongPolicyType $E) {
            IW::getLog()->debug('Acronis: ' . $E->getMessage());
            IW::getLog()->debug('Acronis: attempting to fetch policy of correct type and overwrite id in cache.');

            // overwrite cached id with total protection plan
            // we expect cached id to always be of this type.
            $BackupMachinePolicy = $Api->getBackupMachinePolicy($cache_id);
            $policy_id = $BackupMachinePolicy->getTotalPolicyId();
            $Cache->cachePlanId($policy_id);

            return $Api->getTotalPolicy($policy_id);
        } catch (Exception_Acronis_BackupPlan_Missing $E) {
            IW::getLog()->debug('Acronis: ' . $E->getMessage());
            $Cache->deletePlanId();
        } catch (Exception_Acronis_ApiFailure $E) {
            IW::getLog()->debug('Acronis: Cannot retrieve policy from Acronis Api - must have been deleted.');
            $Cache->deletePlanId();
        }
    }

    /**
     * Get most applicable policy among the policies applied to the resource.
     *
     * Should only use this when none in cache.
     */
    public function getAppliedPolicy()
    {
        $policies = $this->getPoliciesApi()->getAppliedRootPolicies($this->getInstanceId());

        if (empty($policies)) {
            throw new Exception_Acronis_BackupPlan_NoneAssigned("No policy applied to resource!");
        }

        return $this->_getMostApplicableRootPolicy($policies);
    }

    /**
     * Get a policy to create an application from.
     *
     * NOTE: DO NOT USE THIS TO GET POLICY THAT PROTECTS RESOURCE!
     *
     * this is for getting the most suitable policy in case none already is protecting the resource,
     * e.g. when creating application.
     */
    public function getApplicablePolicy(): Model_Backups_Plan_Policy_Total
    {
        $policies = $this->getApplicablePolicies();

        if (empty($policies)) {
            $msg = "No applicable policies exists!";

            if ($this->isProtected()) {
                $msg .= " Plan already activated!";
            }

            throw new Exception_Acronis_BackupPlan_NoApplicablePolicies($msg);
        }

        return $this->_getMostApplicableRootPolicy($policies);
    }

    /**
     *
     * @param Model_Backups_Plan_Policy_Total[] $policies
     *
     * Will return 1) cached 2) default 3) ok pre/post scripts, 4) random
     */
    private function _getMostApplicableRootPolicy(array $policies): Model_Backups_Plan_Policy_Total
    {
        if (empty($policies)) {
            throw new IWorx_Exception_DoingItWrong('"policies" is empty!!! validation most likely missing in the function calling this.');
        }

        // no need for further checks if only one
        if (count($policies) === 1) {
            $Policy = $policies[0];
            return $Policy;
        }

        $Cache = $this->_getCache();
        $cache_id = $Cache->getPlanId();

        if ($Cache->isCached($Cache::CACHE_PLAN_ID)) {
            foreach ($policies as $Policy) {
                if ($Policy->getId() === $cache_id) {
                    return $Policy;
                }
            }
        }

        foreach ($policies as $Policy) {
            if ($Policy->isDefault()) {
                return $Policy;
            }
        }

        foreach ($policies as $Policy) {
            $BackupPlanPolicy = $Policy->getBackupMachinePolicy();

            if ($BackupPlanPolicy && $BackupPlanPolicy->prePostScriptsOK()) {
                return $Policy;
            }
        }

        $msg = "Acronis: most applicable policy does not have pre/post scripts enabled! This might be impairing some of the plugin functionality.";
        IW::getLog()->warning($msg);

        // return first if no default or no valid pre/post scripts.
        $Policy = $policies[0];
        return $Policy;
    }


    /**
     * Get all policies that can be applied to the resource.
     *
     * @return Model_Backups_Plan_Policy_Total[]
     */
    public function getApplicablePolicies(): array
    {
        return $this->getPoliciesApi()->getApplicableRootPolicies($this->getInstanceId());
    }

    /**
     * Get all policies assigned to the resource.
     *
     * @return Model_Backups_Plan_Policy_Total[]
     */
    public function getAppliedPolicies(): array
    {
        return $this->getPoliciesApi()->getAppliedRootPolicies($this->getInstanceId());
    }

    /**
     * Can only create applications with root policy id (type policy.protection.total)
     *
     * @param string $root_policy_id
     */
    public function createApplication($root_policy_id): void
    {
        $Policy = $this->getPoliciesApi()->getTotalPolicy($root_policy_id);
        $policy_id = $Policy->getId();
        $this->getApplicationsApi()->createApplication($policy_id);
        $this->_getCache()->cachePlanId($policy_id);
    }

    /**
     * @param string $root_policy_id
     */
    public function revokeApplication($root_policy_id): void
    {
        $RootPolicy = $this->getPoliciesApi()->getTotalPolicy($root_policy_id);
        $BackupMachinePolicy = $RootPolicy->getBackupMachinePolicy();

        $this->_validateApplicationExists($BackupMachinePolicy);

        $policy_id = $RootPolicy->getId();
        $this->getApplicationsApi()->revokeApplication($policy_id);
    }

    /**
     * get backup status.
     *
     * @return array
     */
    public function getBackupStatus(): array
    {
        $backup_status = array();

        if ($this->isProtected()) {
            $RootPolicy = $this->_requestRootPolicy();
            $policy_id = $RootPolicy->getBackupMachinePolicy()->getId();

            $Application = $this->_getApplication($policy_id);
            $last_run = $Application->getLastRunTime();
            $next_run = $Application->getNextRunTime();
            $status = $Application->getStatus();

            $backup_status[self::LAST_STATUS] = $status;
            $backup_status[self::LAST_BACKUP] = self::formatDate($last_run);
            $backup_status[self::NEXT_BACKUP] = self::formatDate($next_run);
        }

        return $backup_status;
    }

    /**
     * Get application (copy of backup policy - specific to the resource)
     *
     * Includes information such as last run time & current status
     *
     * @return Model_Backups_Plan_Application
     */
    private function _getApplication(): Model_Backups_Plan_Application
    {
        if (!isset($this->_Application)) {
            $Policy = $this->_getBackupMachinePolicy();
            $this->_Application = $this->_requestApplication($Policy);
        }

        return $this->_Application;
    }

    /**
     * Check if backup is in progress.
     *
     * @return boolean
     */
    public function backupInProgress(): bool
    {
        $Policy  = $this->_getBackupMachinePolicy();
        $Application = $this->_requestApplication($Policy);
        return $Application->backupInProgress();
    }

    /**
     * @param Model_Backups_Plan_Policy_BackupMachine $BackupMachinePolicy
     */
    private function _requestApplication(Model_Backups_Plan_Policy_BackupMachine $BackupMachinePolicy): Model_Backups_Plan_Application
    {
        $this->_validateApplicationExists($BackupMachinePolicy);

        $policy_id = $BackupMachinePolicy->getId();
        return $this->getApplicationsApi()->getApplicationByPolicyId($policy_id);
    }

    /**
     * This is the policy including backup settings. Can only retrieve this type if
     * resource is protected
     */
    private function _getBackupMachinePolicy(): Model_Backups_Plan_Policy_BackupMachine
    {
        $RootPolicy = $this->_requestRootPolicy();
        return $RootPolicy->getBackupMachinePolicy();
    }

    /**
     * @param Model_Backups_Plan_Policy_BackupMachine $Policy
     */
    private function _validateApplicationExists(Model_Backups_Plan_Policy_BackupMachine $Policy)
    {
        if (!$Policy->isProtectingResource($this->getInstanceId())) {
            throw new Exception_Acronis_BackupPlan_ApplicationDoesNotExist('getBackupStatus - cannot find application for policy with id: ' . $Policy->getId());
        }
    }

    /**
     * Trigger a backup.
     */
    public function runBackup()
    {
        $Policy = $this->_getBackupMachinePolicy();
        $this->getApplicationsApi()->runBackup($Policy->getId());
    }

    /**
     * Get backup plan details.
     */
    public function getPlanDetails()
    {
        $RootPolicy = $this->getPolicyInUse();
        $protects_resource = $RootPolicy->isProtectingResource($this->getInstanceId());

        $plan_details = array(
            'id' => $RootPolicy->getId(),
            'name' => $RootPolicy->getName(),
            'enabled' => $protects_resource,
        );

        if ($protects_resource) {
            $BackupMachinePolicy = $RootPolicy->getBackupMachinePolicy();

            $plan_details['time'] = $BackupMachinePolicy->getTime();
            $plan_details['days'] = $BackupMachinePolicy->getDays();
        }


        return $plan_details;
    }

    /**
     * Get policy used by the plugin (can be enabled/disabled)
     *
     * @return Model_Backups_Plan_Policy_Total
     */
    public function getPolicyInUse(): Model_Backups_Plan_Policy_Total
    {
        return $this->_requestRootPolicy();
    }

    /**
     * Create default policy
     */
    public function createInterWorxDefaultPolicy()
    {
        $new_plan_id = $this->getPoliciesApi()->createDefaultPolicy();
        $this->_getCache()->cachePlanId($new_plan_id);
    }
}
