<?php

/**
 * InterWorx Hosting Control Panel
 *
 * <pre>
 * +----------------------------------------------------------------------+
 * | Copyright (c) 2000-2010 InterWorx L.L.C., All Rights Reserved.       |
 * +----------------------------------------------------------------------+
 * | Redistribution and use in source form, with or without modification  |
 * | is NOT permitted without consent from the copyright holder.          |
 * |                                                                      |
 * | THIS SOFTWARE IS PROVIDED BY THE AUTHOR AND CONTRIBUTORS "AS IS" AND |
 * | ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO,    |
 * | THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A          |
 * | PARTICULAR PURPOSE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL,    |
 * | EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO,  |
 * | PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR   |
 * | PROFITS; OF BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY  |
 * | OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT         |
 * | (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE    |
 * | USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH     |
 * | DAMAGE.                                                              |
 * +----------------------------------------------------------------------+
 * </pre>
 *
 * @package InterWorx
 * @author  Malin Lorch <mlorch@liquidweb.com>
 * @date    Apr 20, 2020
 */

/**
 * Backup Plan class.
 *
 * @package Interworx
 * @subpackage Plugin_Acronis
 */
class Model_Backups_Plan_Policy_BackupMachine extends Model_Backups_Plan_Policy
{
    const TYPE_PRE_DATA_COMMAND = 'pre_data_command';
    const TYPE_POST_DATA_COMMAND = 'post_data_command';

    private $_hour;
    private $_minute;
    private $_weekdays;
    private $_pre_post_commands;
    private $_schedule_type;
    private $_total_policy_id;

    /**
     * Different from parent, in that it requires a context object!
     *
     * This is because this type of policy always has a context object; this is the type
     * that specifies backup schedule etc for resource machine.
     *
     * @param array $policies
     * @param array $context
     */
    public function __construct(array $policies, array $context = null)
    {
        $policy = $this->_getPolicyByType($policies, self::TYPE_POLICY_BACKUP_MACHINE);

        if (empty($policy)) {
            throw new Exception_Acronis_BackupPlan_WrongPolicyType('wrong type');
        }

        $policy_total = $this->_getPolicyByType($policies, self::TYPE_POLICY_PROTECTION_TOTAL);

        if (empty($policy_total)) {
            throw new Exception_Acronis_BackupPlan_Missing('backup machine policy should always include its parent!!');
        }

        // do not create parent class here - it will create this class, which
        // results in endless loop!
        $this->_total_policy_id = $policy_total['id'];

        parent::__construct($policy, $context);

        $settings = $this->_getSettings();
        $schedule = $settings['scheduling']['backup_sets'][0]['schedule'];
        $alarms = $schedule['alarms'];
        $time = $alarms['time'];
        $repeat_at = $time['repeat_at'][0];

        $this->_schedule_type = $schedule['type']; // daily or weekly
        $this->_weekdays = $time['weekdays']; // this does not exist -- should it? exists for other types...


        $this->_pre_post_commands = $settings['pre_post_commands'];
        $this->_hour = $repeat_at['hour'];
        $this->_minute = $repeat_at['minute'];
    }

    /**
     * Days is not always set in the api response -- only handling empty response if daily (assuming every day)
     *
     * @return array
     */
    public function getDays()
    {
        if (!isset($this->_weekdays) && $this->_schedule_type === 'daily') {
            // need to return all days if days not returned by api
            // for backward compatibility (old api would return this)
            $this->_weekdays = array(
                'mon',
                'tue',
                'wed',
                'thu',
                'fri',
                'sat',
                'sun',
            );
        }

        return $this->_weekdays;
    }

    /**
     * Get time
     *
     * @return array
     */
    public function getTime(): array
    {
        return array(
            'hour' => $this->_hour,
            'minute' => $this->_minute,
            'type' => $this->_schedule_type
        );
    }

    /**
     * @return array
     */
    public function getPreCommands(): array
    {
        return $this->_getCommandByType(self::TYPE_PRE_COMMAND);
    }

    /**
     * @return array
     */
    public function getPreDataCommands(): array
    {
        return $this->_getCommandByType(self::TYPE_PRE_DATA_COMMAND);
    }

    /**
     * @return array
     */
    public function getPostCommands(): array
    {
        return $this->_getCommandByType(self::TYPE_POST_COMMAND);
    }

    /**
     * @param string $type
     * @return array
     */
    private function _getCommandByType($type): array
    {
        foreach ($this->_getPrePostCommands() as $command) {
            if ($command['type'] === $type) {
                return $command;
            }
        }

        IW::getLog()->debug("Acronis: " . __FUNCTION__ . ": command settings does not seem to exist");
        IW::getLog()->debug($this->_getSettings());
        return array();
    }

    /**
     * get pre/post commands
     */
    private function _getPrePostCommands(): array
    {
        if (!isset($this->_pre_post_commands)) {
            IW::getLog()->debug("Acronis: " . __FUNCTION__ . ": no pre_post_commands found");
            return array();
        }

        return $this->_pre_post_commands;
    }

    /**
     * Check if pre/post scripts are ok.
     *
     * @return boolean
     */
    public function prePostScriptsOK()
    {
        $enabled = $this->_prePostCommandsEnabled();
        $pre_valid = $this->_preCommandConfigOK();
        $post_valid = $this->_postCommandConfigOK();
        return $enabled && $pre_valid && $post_valid;
    }

    /**
     * Check if pre/post commands are enabled.
     *
     * @return boolean
     */
    private function _prePostCommandsEnabled(): bool
    {
        return $this->getPreCommands()['enabled'] && $this->getPostCommands()['enabled'];
    }

    /**
     * Check if existing plan has right configuration.
     *
     * @return boolean
     */
    private function _preCommandConfigOK()
    {
        $expected_config = $this->_getExpectedConfig(self::PRE_SCRIPT_CONFIG_CMD);
        return $this->_configOk($this->getPreCommands(), $expected_config);
    }

    /**
     * Check if existing plan has right configuration.
     *
     * @return boolean
     */
    private function _postCommandConfigOK()
    {

        $expected_config = $this->_getExpectedConfig(self::POST_SCRIPT_CONFIG_CMD);
        return $this->_configOk($this->getPostCommands(), $expected_config);
    }

    /**
     * Get expected config to compare actual against.
     *
     * @param string $expected_cmd
     */
    private function _getExpectedConfig($expected_cmd): array
    {
        return array(
            'workdir' => "''",
            'stop_on_error' => true,
            'args' => '{RESOURCE_ID}',
            'wait' => true,
            'cmd' => $expected_cmd,
        );
    }

    /**
     * Check if config matches expectations.
     *
     * @param array $actual_config
     * @param array $expected_config
     * @return boolean
     */
    private function _configOk(array $actual_config, array $expected_config): bool
    {
        foreach ($expected_config as $key_expected => $value_expected) {
            if ($actual_config[$key_expected] !== $value_expected) {
                return false;
            }
        }

        return true;
    }

    /**
     * Get parent policy id (policy of type policy.protection.total also referred to as "root" policy)
     *
     * @return string
     */
    public function getTotalPolicyId(): string
    {
        return $this->_total_policy_id;
    }
}
