<?php

/**
 * InterWorx Hosting Control Panel
 *
 * <pre>
 * +----------------------------------------------------------------------+
 * | Copyright (c) 2000-2010 InterWorx L.L.C., All Rights Reserved.       |
 * +----------------------------------------------------------------------+
 * | Redistribution and use in source form, with or without modification  |
 * | is NOT permitted without consent from the copyright holder.          |
 * |                                                                      |
 * | THIS SOFTWARE IS PROVIDED BY THE AUTHOR AND CONTRIBUTORS "AS IS" AND |
 * | ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO,    |
 * | THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A          |
 * | PARTICULAR PURPOSE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL,    |
 * | EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO,  |
 * | PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR   |
 * | PROFITS; OF BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY  |
 * | OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT         |
 * | (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE    |
 * | USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH     |
 * | DAMAGE.                                                              |
 * +----------------------------------------------------------------------+
 * </pre>
 *
 * @package InterWorx
 * @author  Malin Lorch <mlorch@liquidweb.com>
 * @date    Aug 28, 2020
 */

/**
 *
 * @package Interworx
 * @subpackage Plugin_Acronis
 */
class Model_Recovery_Files_Databases_Nodeworx extends Model_Recovery_Files_Databases
{

    /**
     * Get dbs from info cached in prescript.
     * Defaults to parsing api if prescript info
     * was not collected at the time of the backup.
     *
     * @return array
     */
    public function getDbs()
    {
        $Cache = $this->_getBackupCache();
        $id = $this->getBackupId();

        if ($Cache->prescriptCached($id)) {
            return $this->_getDbsFromPrescript();
        }

        // uses old cache system - retrieves db names from
        // acronis api by parsing folder names
        return $this->_getDbsNoPrescript();
    }

    /**
     * Get databases cached at backup prescript.
     *
     * @return array
     */
    private function _getDbsFromPrescript()
    {
        $swDbs = $this->_mapPrescriptDbsToSwUsers();
        $unmapped = $this->_mapUnmappedDbs();

        return array_merge(array($unmapped), $swDbs);
    }

    /**
     * Map prescript databases to sw user.
     */
    private function _mapPrescriptDbsToSwUsers()
    {
        return array_map(function ($user) {
            return $this->_getSwUserDbsFromPrescript($user);
        }, $this->_getSwUsersFromBackupCache());
    }

    /**
     * @param string $user
     * @return array
     */
    private function _getSwUserDbsFromPrescript($user)
    {
        $backupId = $this->getBackupId();
        $Cache = $this->_getBackupCache();
        $dbs = $Cache->getUserDatabases($backupId, $user);

        return array(
            self::DUMP_INFO_USER => $user,
            self::DUMP_INFO_DBS => $this->_getDbNames($dbs)
        );
    }


    /**
     * @return array
     */
    private function _getSwUsersFromBackupCache()
    {
        $backupId = $this->getBackupId();
        $Cache = $this->_getBackupCache();
        return $Cache->getSwUsers($backupId);
    }


    /**
     * Map unmapped databases to "unixuser" for consistent
     * output (sw/unmapped should display is similar way).
     */
    private function _mapUnmappedDbs()
    {
        $Cache = $this->_getBackupCache();
        $dbs = $Cache->getUnmappedDatabases($this->getBackupId());

        return array(
            self::DUMP_INFO_USER => 'UNMAPPED',
            self::DUMP_INFO_DBS => $this->_getDbNames($dbs)
        );
    }

    /**
     * Get dbs by parsing folders in acronis api response.
     *
     * @return array
     */
    private function _getDbsNoPrescript()
    {
        $Cache = $this->_getCache();
        $dbInfo = $Cache->getDumpsInfo($this->getBackupId());

        if (!isset($dbInfo)) {
            $this->_cacheDumpsFromApi($this->getBackupId(), $dbInfo);
            return $this->_getDbsNoPrescript();
        }

        return $dbInfo;
    }

    /**
     * Restore dbs. If unix user does not exist, it will
     * restore databases as unmapped.
     *
     * WARNING - this might take up to an hour to execute.
     * Execute in background is recommended.
     *
     * @param array  $dbs
     * @param string $unixUser
     */
    public function restoreSiteworxDatabases(array $dbs, $unixUser)
    {
        $this->_restoreDumps($dbs, $unixUser, $unixUser);
        $this->_restoreUserDatabases($unixUser, $dbs);
    }

    /**
     * Restore dumps.
     *
     * Foldername is either unixuser or folder name specified
     * in prescript for unmapped databases.
     *
     * @param array  $dbs
     * @param string $foldername
     * @param string $user
     */
    private function _restoreDumps(array $dbs, $foldername, $user)
    {
        $this->_addToRestoreCache($dbs, $user);
        $this->_logStartingRestore($dbs);
        $this->_restoreUserDumps($foldername);
        $this->_awaitUserDumpsRestoration($foldername);
    }



    /**
     * @param array $dbs
     */
    public function restoreUnmappedDatabases(array $dbs)
    {
        $Backup = $this->_getBackup();
        $dumpsFolder = $Backup->getUnmappedDbFolder();

        $this->_restoreDumps($dbs, $dumpsFolder, self::UNMAPPED);
        $this->_restoreUnmappedDatabases($dbs);
    }

    /**
     * @param boolean $dbsUnmapped
     */
    private function _logStartingRestore($dbsUnmapped)
    {
        $type = $dbsUnmapped ? 'unmapped' : 'SW';
        IW::getLog()->info("Acronis: starting $type db restore");
    }

    /**
     * restore user databases.
     * Will restore dbs as unmapped if unix user does not exist.
     *
     * @param string $user
     * @param array  $dbs
     */
    private function _restoreUserDatabases($user, array $dbs)
    {
        try {
            $this->_restoreSwDatabases($user, $dbs);
        } catch (Exception_Acronis_AccountDoesNotExist $e) {
            $this->_setFailed('failed to import db to SW account. Will continue by importing unmapped database');
            $this->_restoreUnmappedDatabases($dbs);
        }
    }

    /**
     * @param string $user
     * @param array  $databases
     */
    private function _restoreSwDatabases($user, array $databases)
    {
        $SwRecoveryPointDatabases = new Model_Recovery_Files_Databases_Siteworx($this->getBackupId(), $user, $this->getRestoreId());
        $SwRecoveryPointDatabases->restoreDatabases($databases);
    }

    /**
     * Restore unmapped dbs.
     *
     * @param array $dbs
     */
    private function _restoreUnmappedDatabases(array $dbs)
    {
        IW::getLog()->info('Acronis: unmapped database restoration in progress');
        $this->_setInProgress();

        $Backup = $this->_getBackup();
        $dumpsFolder = $Backup->getUnmappedDbFolder();

        foreach ($dbs as $dbName) {
            $dumpsPath = $this->_getRestoredDumpsPath($dumpsFolder) . "/$dbName";
            $this->_unzipUnmappedDatabaseArchive($dumpsPath, $dbName);

            $sqlFiles = $this->_getSqlFiles($dumpsPath, $dbName);
            $this->_restoreUnmappedDatabase($dbName, $sqlFiles);
        }

        $this->_setComplete();
    }


    /**
     * unzip archives.
     *
     * @param string $dumpsPath
     * @param string $dbName
     */
    private function _unzipUnmappedDatabaseArchive($dumpsPath, $dbName)
    {
        if ($dbName !== 'mysql') {
            $archives = $this->_getTableArchives($dumpsPath);
            $this->_unzipArchivesUnmappedDb($archives);
        }
    }

    /**
     * Unzip archives in current destination.
     * Only use for unmapped databases, as sw dumps should be placed in sw folder.
     *
     * @param array $archivePaths
     */
    private function _unzipArchivesUnmappedDb(array $archivePaths)
    {
        foreach ($archivePaths as $archive) {
            $destination = $this->_getArchiveDestination($archive);
            $this->_uncompress($archive, $destination);
        }
    }

    /**
     * @param string $archive
     */
    private function _getArchiveDestination($archive): string
    {
        $extLength = 3;
        return substr($archive, 0, strlen($archive) - $extLength);
    }

    /**
     * restore unmapped db to local db.
     *
     * @param string $dbName
     * @param array  $sqlFiles
     */
    private function _restoreUnmappedDatabase($dbName, array $sqlFiles)
    {
        if (empty($sqlFiles)) {
            $this->_setFailed('Sql dump files do not exist for database: ' . $dbName);
            throw new Exception('Acronis: Cannot restore databases - sql files missing');
        }

        $Importer = new Model_Recovery_Files_Databases_UnmappedDbImporter($dbName, $sqlFiles);
        $Importer->recoverDatabase();
    }

    /**
     * @param string $dirName
     * @param array  $dbNames
     */
    public function restoreDatabasesAltLocation($dirName, array $dbNames)
    {
        // YYYY-MM-DD-MM-SS
        $dateDirName = date("Y-m-d-H-i-s");
        $destination = self::NW_DOWNLOAD_DIR_PATH . "/$dateDirName/$dirName";
        $this->_restoreUserDbsToAltLocation($dirName, $dbNames, $destination);
    }
}
