<?php
/**
 * InterWorx Hosting Control Panel
 *
 * <pre>
 * +----------------------------------------------------------------------+
 * | Copyright (c) 2000-2021 InterWorx L.L.C., All Rights Reserved.       |
 * +----------------------------------------------------------------------+
 * | Redistribution and use in source form, with or without modification  |
 * | is NOT permitted without consent from the copyright holder.          |
 * |                                                                      |
 * | THIS SOFTWARE IS PROVIDED BY THE AUTHOR AND CONTRIBUTORS "AS IS" AND |
 * | ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO,    |
 * | THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A          |
 * | PARTICULAR PURPOSE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL,    |
 * | EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO,  |
 * | PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR   |
 * | PROFITS; OF BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY  |
 * | OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT         |
 * | (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE    |
 * | USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH     |
 * | DAMAGE.                                                              |
 * +----------------------------------------------------------------------+
 * </pre>
 *
 * @package    InterWorx
 * @author     Malin Lorch <mlorch@liquidweb.com>
 * @date       Sep 20, 2021
 */
require_once(__DIR__ . '/../../Siteworx.php');
/**
 * @package InterWorx
 * @subpackage
 */
class SwGetContentTest extends Acronis_SiteworxTest
{
    const NEW_TEST_USER = 'newtestuser';

    /**
     * Create sw accounts if it does not exist.
     */
    public function setUp()
    {
        if (!$this->_getSw()->exists()) {
            $this->_createSwAccount();
        }

        if (!$this->_getSw(self::NEW_TEST_USER)->exists()) {
            $this->_createSwAccount(self::NEW_TEST_USER);
        }
    }

    /**
     * Delete test user after tests.
     */
    public function tearDown()
    {
        $this->_deleteSwAccount(self::NEW_TEST_USER);
    }

    /**
     * test getRootContent - compares content collected before backup with acronis api res.
     */
    public function testGetRootContent()
    {
        $expectedRootContentAcronisTest = $this->_getRootContent(self::ACRONISTESTUSER);
        $expectedDomainContentAcronisTest = $this->_getDomainContent(self::ACRONISTESTUSER);

        $expectedRootContentNewUser = $this->_getRootContent(self::NEW_TEST_USER);
        $expectedDomainContentNewUser = $this->_getDomainContent(self::NEW_TEST_USER);

        $this->_runBackup();

        $this->_testGetRoot(self::ACRONISTESTUSER, $expectedRootContentAcronisTest);
        $this->_testGetRoot(self::NEW_TEST_USER, $expectedRootContentNewUser);

        $this->_testGetDirectoryContent(self::ACRONISTESTUSER, $this->_getDomainPath(self::ACRONISTESTUSER), $expectedDomainContentAcronisTest);
        $this->_testGetDirectoryContent(self::NEW_TEST_USER, $this->_getDomainPath(self::NEW_TEST_USER), $expectedDomainContentNewUser);
    }

    /**
     * Get domain path.
     *
     * @param string $user
     * @return string
     */
    private function _getDomainPath($user)
    {
        return "/$user.com";
    }

    /**
     * @param string $user
     * @return array
     */
    private function _getDomainContent($user)
    {
        $root = $this->_getRoot($user);
        return Model_AcronisFileSys::getDirContent("$root/$user.com");
    }

    /**
     * @param string $user
     * @return array
     */
    private function _getRootContent($user)
    {
        $path = $this->_getRoot($user);
        return Model_AcronisFileSys::getDirContent($path);
    }


    /**
     * Compare content from  getRootContent
     *
     * @param string $user
     * @param array  $expectedContent
     */
    private function _testGetRoot($user, array $expectedContent)
    {
        $Backup = $this->_getBackup();
        $args = array(Form_AcronisBackups::INPUT_BACKUP_ID => $Backup->getBackupId());

        $Reply = $this->_sendCtrlRequest('getRootContent', $args, $user);
        $actualRootContent = $Reply->getPayloadData();

        $SW = $this->_getSW($user);
        $root = $SW->getPackageRoot();

        // acronis will not include 'var' due to limited access
        $expectedCount = count($expectedContent) - 1;

        $this->assertSame($expectedCount, count($actualRootContent));
        $this->_compareContent($expectedContent, $actualRootContent);
        $this->_assertParentSame($actualRootContent, $root);
    }

    /**
     * @param string $user
     * @param string $path
     * @param array  $expectedContent
     */
    private function _testGetDirectoryContent($user, $path, array $expectedContent)
    {
        $Backup = $this->_getBackup();
        $args = array(
            Form_AcronisBackups::INPUT_BACKUP_ID => $Backup->getBackupId(),
            Form_AcronisBackups::INPUT_DIRECTORY_PATH => $path
        );

        $Reply = $this->_sendCtrlRequest('getDirectoryContent', $args, $user);

        $content = $Reply->getPayloadData();
        $this->_compareContent($expectedContent, $content);
    }

    /**
     * @param string $user
     * @return string
     */
    private function _getRoot($user)
    {
        $SW = $this->_getSw($user);
        return $SW->getPackageRoot();
    }

    /**
     * Compare content prebackup / acronis
     *
     * @param array $expected
     * @param array $actual
     */
    private function _compareContent(array $expected, array $actual)
    {
        foreach ($actual as $dir) {
            $this->assertContains($dir['name'], $expected);
        }
    }

    /**
     * @param array  $content
     * @param string $expectedParent
     */
    private function _assertParentSame(array $content, $expectedParent)
    {
        foreach ($content as $dir) {
            $this->assertSame($this->_getParentPath($dir['path']), $expectedParent);
        }
    }

    /**
     * @param string $path
     * @return string
     */
    private function _getParentPath($path)
    {
        $pathParts = explode('/', $path);

        // exclude current dir
        array_pop($pathParts);
        // exclude device
        array_shift($pathParts);

        return "/" . implode('/', $pathParts);
    }
}
