<?php
/**
 * InterWorx Hosting Control Panel
 *
 * <pre>
 * +----------------------------------------------------------------------+
 * | Copyright (c) 2000-2021 InterWorx L.L.C., All Rights Reserved.       |
 * +----------------------------------------------------------------------+
 * | Redistribution and use in source form, with or without modification  |
 * | is NOT permitted without consent from the copyright holder.          |
 * |                                                                      |
 * | THIS SOFTWARE IS PROVIDED BY THE AUTHOR AND CONTRIBUTORS "AS IS" AND |
 * | ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO,    |
 * | THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A          |
 * | PARTICULAR PURPOSE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL,    |
 * | EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO,  |
 * | PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR   |
 * | PROFITS; OF BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY  |
 * | OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT         |
 * | (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE    |
 * | USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH     |
 * | DAMAGE.                                                              |
 * +----------------------------------------------------------------------+
 * </pre>
 *
 * @package    InterWorx
 * @author     Malin Lorch <mlorch@liquidweb.com>
 * @date       Sep 20, 2021
 */
require_once(__DIR__ . '/../../Siteworx.php');
/**
 * @package InterWorx
 * @subpackage
 */
class SwRecoverDirectoriesTest extends Acronis_SiteworxTest
{
    const NEWDIR = 'NEWDIR';

    /**
     * delete initiated flag after each test.
     */
    public function tearDown()
    {
        $this->_deleteFlagInitiatedIfExists();
    }

    /**
     * test recover directories - should not delete new content
     */
    public function testRecoverDirectories()
    {
        echo __FUNCTION__;

        $user = self::ACRONISTESTUSER;
        $parent = self::NEWDIR;
        $shouldBeRestored = "$parent/someohterdir";

        $this->_createUserDir($user, $shouldBeRestored);
        $this->_runBackup();
        $this->_deleteUserDir($user, $shouldBeRestored);

        // new content should not be deleted when restoring parent
        $shouldNotBeDeleted = "$parent/DELETEMENOT";
        $this->_createUserDir($user, $shouldNotBeDeleted);

        $deleteNewContent = false;
        $Reply = $this->_recoverDirectories(array($parent), $deleteNewContent);
        $this->assertReplySuccessful($Reply);

        $payloadData = $Reply->getPayloadData();
        $planId = $payloadData['planId'];
        $this->_awaitRestore($planId);

        $this->assertTrue($this->_isUserDir($user, $shouldBeRestored));
        $this->assertTrue($this->_isUserDir($user, $shouldNotBeDeleted));

        return $planId;
    }

    /**
     * @depends testRecoverDirectories
     * @param string $planId
     */
    public function testListRecoveries($planId)
    {
        echo __FUNCTION__;

        $Reply = $this->_sendCtrlRequest('listRecoveries');
        $this->assertReplySuccessful($Reply);

        // assume latest restore is first item
        $payloadData = $Reply->getPayloadData();
        $restore = array_pop($payloadData);
        $this->assertSame(strtoupper($planId), strtoupper($restore['planId']));
    }

    /**
     * test recoverDirectories - should delete new content
     */
    public function testRecoverDirectoriesDeleteNewContent()
    {
        echo __FUNCTION__;

        $user = self::ACRONISTESTUSER;
        $parent = self::NEWDIR;
        $shouldBeRestored = "$parent/someohterdir";

        $this->_createUserDir($user, $shouldBeRestored);
        $this->_runBackup();
        $this->_deleteUserDir($user, $shouldBeRestored);

        // new content should be deleted when restoring parent
        $shouldBeDeleted = "$parent/DELETEME";
        $this->_createUserDir($user, $shouldBeDeleted);

        $deleteNewContent = true;
        $Reply = $this->_recoverDirectories(array($parent), $deleteNewContent);
        $payloadData = $Reply->getPayloadData();

        $planId = $payloadData['planId'];
        $this->_awaitRestore($planId);

        $this->assertTrue($this->_isUserDir($user, $shouldBeRestored));
        $this->assertFalse($this->_isUserDir($user, $shouldBeDeleted));
    }

    /**
     * @param string $user
     * @param string $dir
     * @return boolean
     */
    private function _isUserDir($user, $dir)
    {
        $absolutePath = $this->_getSwAbsolutePath($user, $dir);
        return Model_AcronisFileSys::isDir($absolutePath);
    }

    /**
     * create user dir.
     *
     * @param string $user
     * @param string $dir
     */
    private function _createUserDir($user, $dir)
    {
        $absolutePath = $this->_getSwAbsolutePath($user, $dir);
        Model_AcronisFileSys::createDir($absolutePath);
        $this->assertTrue(Model_AcronisFileSys::isDir($absolutePath), "Failed asserting that created directory exists!!!");
    }

    /**
     * @param string $user
     * @param string $dir
     */
    private function _deleteUserDir($user, $dir)
    {
        $absolutePath = $this->_getSwAbsolutePath($user, $dir);
        Model_AcronisFileSys::rm($absolutePath);
        $this->assertFalse(Model_AcronisFileSys::isDir($absolutePath));
    }

    /**
     * test recoverDirectories - should fail if flag 'initiated' is set
     */
    public function testRecoverDirectoriesShouldFailIfRestoreInitiated()
    {
        IW::ENV()->setEnvFlag(Model_Recovery_Files::ENV_FLAG_RESTORE_INITIATED);

        $Reply = $this->_recoverDirectories(array(self::NEWDIR));
        $this->assertReplyNotSuccessful($Reply, 'expected recovery to fail when restore initiated');
        $this->_assertReplyReturnsRestoreInProgress($Reply);

        IW::ENV()->clearEnvFlag(Model_Recovery_Files::ENV_FLAG_RESTORE_INITIATED);
    }
}
